/*
 * Uloha1Test.java
 * JUnit based test
 *
 */
package uloha1;

import java.io.IOException;
import java.net.URI;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.dom.Document;

/**
 *
 * @author Petr Adamek, changes by Tomas Pitner, Feb 2008
 */
public class Uloha1Test extends TestCase {

    public Uloha1Test(String testName) {
        super(testName);
    }
    Uloha1 uloha1;
    XPath xpath;

    protected void setUp() throws Exception {
        URI uri = Uloha1Test.class.getResource("people.xml").toURI();
        if (uri == null) {
            throw new IOException("Nelze nalezt testovaci XML soubor");
        }
        uloha1 = Uloha1.newInstance(uri);

        xpath = XPathFactory.newInstance().newXPath();
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(Uloha1Test.class);
        return suite;
    }

    /**
     * Test of changeName method, of class uloha1.Uloha1.
     */
    public void testChangeName() throws XPathExpressionException {
        uloha1.changeName("Hroza", "Honzik");
        assertEquals("Honzik", xpath.evaluate(
                "//person[@id=3]/name", uloha1.getDocument()));

        uloha1.changeName("Novák", "Pavlík");
        assertEquals("Pavlík", xpath.evaluate(
                "//person[@id=1]/name", uloha1.getDocument()));

        uloha1.changeName("Hroza", "Honzíček");
        assertEquals("Honzíček", xpath.evaluate(
                "//person[@id=3]/name", uloha1.getDocument()));
    }

    /**
     * Test of getWageCosts method, of class uloha1.Uloha1.
     */
    public void testGetAverageCarAge() throws XPathExpressionException {
        assertEquals(getAverageCarAge(uloha1.getDocument()), uloha1.getAverageCarAge());
    }

    private double getAverageCarAge(Document doc) throws XPathExpressionException {
        return ((Double) xpath.evaluate(
                "sum(//year)", uloha1.getDocument(), XPathConstants.NUMBER)) / ((Double) xpath.evaluate(
                "count(//year)", uloha1.getDocument(), XPathConstants.NUMBER));
    }

    public void testRemoveCarsOlderThan1990() throws XPathExpressionException {
        assertEquals("There should be just one car older than 1990 removed.",
                1, uloha1.removeCarsProducedEarlierThan(1990));
        assertEquals("3 cars older than 2002 should be present after removal.",
                3, getOlderCarsCount(2002));
    }

    public void testRemoveCarsOlderThan1996() throws XPathExpressionException {
        assertEquals(3, uloha1.removeCarsProducedEarlierThan(1996));
        assertEquals("1 car older than 2002 should be present after removal.",
                1, getOlderCarsCount(2002));
    }

    private int getOlderCarsCount(int year) throws XPathExpressionException {
        double carsWithoutYear = (Double) xpath.evaluate("count(//car[not(year)])",
                uloha1.getDocument(), XPathConstants.NUMBER);
        if(carsWithoutYear > 0) {
            fail("You have just removed year child element, not the whole car!");
        }
        double older = (Double) xpath.evaluate("count(//car[year<" + year + " and count(year)=1])",
                uloha1.getDocument(), XPathConstants.NUMBER);
        return (int) older;
    }
}
